import os.path
import pickle

from org.bukkit.event import EventPriority
from org.bukkit.event.player import PlayerJoinEvent
from org.bukkit.event.player import PlayerRespawnEvent
from org.bukkit.event.player import PlayerInteractEvent
from org.bukkit.event.entity import EntityDamageByEntityEvent
from org.bukkit.event.entity.EntityDamageEvent import DamageCause
from org.bukkit.inventory import ItemStack
from org.bukkit.entity import EntityType
from org.bukkit.entity import Projectile
from org.bukkit.entity import Player

global highscore

def getHighscoreForPlayer(player):
    global highscore
    for index in range(len(highscore)):
        if player.getUniqueId().toString() == highscore[index].getUniqueId():
            return highscore[index]
    #noch kein highscore vorhanden
    highscore.append(Highscore(player.getUniqueId().toString(), player.getName()))
    return highscore[len(highscore)-1]

class Highscore:
    def __init__(self, uniqueId, name):
        self.__uniqueId = uniqueId
        self.__name = name
        self.__punkte = 0

    def getUniqueId(self):
        return self.__uniqueId

    def getName(self):
        return self.__name

    def getPunkte(self):
        return self.__punkte

    def plusPunkt(self):
        self.__punkte = self.__punkte + 1

    def minusPunkt(self):
        self.__punkte = self.__punkte - 1

class MasterListener(PythonListener):
    def schneebaelleGeben(self, event):
        spieler = event.getPlayer()
        inventar = spieler.getInventory()

        if inventar.contains(bukkit.Material.SNOW_BALL):
            #Spieler hat bereits mindestens einen Schneeball
            inventar.getItem(inventar.first(bukkit.Material.SNOW_BALL)).setAmount(16)
        else:
            #Spieler hat noch keinen Schneeball
            if inventar.firstEmpty() < 0 | inventar.firstEmpty() > 35:
                #Inventar voll
                inventar.remove(inventar.getItem(0))
            inventar.addItem(ItemStack(bukkit.Material.SNOW_BALL, 16))

class JoinListener(MasterListener):
    @PythonEventHandler(PlayerJoinEvent, EventPriority.NORMAL) 
    def onEvent(self, event):
        self.schneebaelleGeben(event)

class RespawnListener(MasterListener):
    @PythonEventHandler(PlayerRespawnEvent, EventPriority.NORMAL) 
    def onEvent(self, event):
        self.schneebaelleGeben(event)

class InteractListener(PythonListener):
    @PythonEventHandler(PlayerInteractEvent, EventPriority.NORMAL) 
    def onEvent(self, event):
        if event.hasItem():
            item = event.getItem()
            if item.getType() == bukkit.Material.SNOW_BALL:
                item.setAmount(item.getAmount())

class HitListener(PythonListener):
    @PythonEventHandler(EntityDamageByEntityEvent, EventPriority.NORMAL)
    def onEvent(self, event):
        if event.getCause() == DamageCause.PROJECTILE:
            projektil = event.getDamager()
            if projektil.getType() == EntityType.SNOWBALL:
                #geschoss ist schneeball
                angreifer = projektil.getShooter()
                angegriffener = event.getEntity()
                if isinstance(angreifer, Player) & isinstance(angegriffener, Player):
                    #punkte verteilen
                    highAngreifer = getHighscoreForPlayer(angreifer)
                    highAngreifer.plusPunkt()
                    highAngegriffener = getHighscoreForPlayer(angegriffener)
                    highAngegriffener.minusPunkt()

class SchneeballschlachtPlugin(PythonPlugin):
    def onEnable(self):
        global highscore
        pfad = self.getDataFolder().getPath() + "/ highscore.pkl"

        if os.path.isfile(pfad):
            datei = open(pfad, "rb")
            highscore = pickle.load(datei)
        else:
            highscore = []

        pluginManager = self.getServer().getPluginManager()
        jListener = JoinListener()
        rListener = RespawnListener()
        iListener = InteractListener()
        hListener = HitListener()
        pluginManager.registerEvents(jListener, self)
        pluginManager.registerEvents(rListener, self)
        pluginManager.registerEvents(iListener, self)
        pluginManager.registerEvents(hListener, self)
        pass

    def onDisable(self):
        global highscore

        if len(highscore) > 0:
            if not self.getDataFolder().exists():
                self.getDataFolder().mkdirs()

            pfad = self.getDataFolder().getPath() + "/highscore.pkl"
            datei = open(pfad, "wb")
            pickle.dump(highscore, datei)

    def onCommand(self, sender, command, label, args):
        if isinstance(sender, Player):
            global highscore
            highscore.sort(key = lambda x: x.getPunkte, reverse=True)
            for index in range(len(highscore)):
                sender.sendMessage(str(index+1) + ". " + highscore[index].getName() + ", " + str(highscore[index].getPunkte()) + " Punkte")